# -*- Mode: Perl -*-
#
#   This file contains the modules that will search for and verify anomalies 
#   in log lines passed from the main engine.  Each module is a function 
#   that should return "0" for condition not met, or a non-zero for 
#   condition met.


# This function will check for somebody attempting to telnet
# directly in as root.  Since root login via a telnet connection
# is disabled by default, we should be able to just look for a 
# failed login for root

sub mod_root_telnet_in {
	(my @retval=());
	(my %tmphash=());

	%tmphash=&head_parse($logs[$_[0]]{'logline'});

	if($tmphash{'process'} eq 'login') {
		if($logs[$_[0]]{'logfile'} eq '/var/log/secure') {
			if($logs[$_[0]]{'logline'}=~/.*?FAILED LOGIN (.*?) FROM (.*?) FOR root, Authen/) {
				$retval[0]=$2;
			}
		}
	}
	return @retval;
}

# This function will check for somebody attempting to ftp
# directly in as root.  Since root ftp is disabled by default
# (in /etc/ftpusers), they should be blocked AND logged.  ;)

sub mod_root_ftp_in {
	(my @retval=());
	(my %tmphash=());

	%tmphash=&head_parse($logs[$_[0]]{'logline'});

	if($tmphash{'process'} eq 'ftpd') {
		if($logs[$_[0]]{'logfile'} eq '/var/log/messages') {
			if($logs[$_[0]]{'logline'}=~/.*?failed login from (.*?), (.*?)$/) {
				$retval[0]=$1;
			} 
		} 
	}
	return @retval;
}

# This function will check for multiple failed su attempts.
# Conditions for trigger: same user has > 2 failed su attempts
# within a 60 second time frame.
#
# To change the trigger, modify $max_failed_attempts and $time_frame
sub mod_failed_multiple_su {
	(my @retval=());
	(my %tmphash=());
	(my %targets=());
	(my $i);
	(my $numtries=1);
	(my $eviluser="");
	(my $victim="");
	(my $keys="");
	(my @keys=());
	(my $ep);
	(my $newep);

	(my $max_failed_attempts = 2);
	(my $time_frame = 60);

	%tmphash=&head_parse($logs[$_[0]]{'logline'});

	if(($logs[$_[0]]{'logfile'} eq '/var/log/messages') && ($tmphash{'process'} eq 'PAM_pwdb')) {
		if($logs[$_[0]]{'logline'} =~ /.*?authentication failure; (.*?)\(.*?-> (.*?) for/) {
			$eviluser=$1; $targets{$2}=1; $victim=$tmphash{'host'};
			$ep=&to_epoch($logs[$_[0]]{'logline'});
			$newep=$ep+1;
			for($i=$_[0];($ep-$newep)<$time_frame && $i>=0;$i--) {
			    if ($ep >= $newep) {
				%tmphash=&head_parse($logs[$i]{'logline'});
				if(($logs[$i]{'logfile'} eq '/var/log/messages')&&($tmphash{'process'} eq 'PAM_pwdb')){
					if($logs[$i]{'logline'}=~/.*?authentication failure; (.*?)\(.*?-> (.*?) for/) {
						if($1 eq $eviluser && $tmphash{'host'} eq $victim) {
							$targets{$2}=1; $numtries++;
						}
					}
				} 
			    }
			if ($logs[$i-1]{'logfile'} eq $logs[$_[0]]{'logfile'})
			    { $newep=&to_epoch($logs[$i-1]{'logline'}); }
			}
			@keys=keys(%targets);
			for($i=0;$keys[$i];$i++) {
				$keys.="$keys[$i],";
			}
			chop($keys);
			if ($numtries> $max_failed_attempts) {
				@retval=($eviluser,$keys);
			} 
		} 
	}
	return @retval;
}

# This function will check for multiple failed incoming telnet attempts.
# Conditions for trigger: same host has > 2 failed telnet attempts
# within a 20 second time frame.
#
# To change trigger conditions, modify $max_failed_attempts and $time_frame
sub mod_failed_multiple_telnets {
	(my @retval=());
	(my %tmphash=());
	(my $evilhost="");
	(my %targets=());
	(my $ep="");
	(my $victim="");
	(my $newep="");
	(my $numtries=1);
	(my @keys=());
	(my $keys="");
	(my $i);

	(my $max_failed_attempts = 2);
	(my $time_frame = 20);

	%tmphash=&head_parse($logs[$_[0]]{'logline'});

	if($tmphash{'process'} eq 'syslog') {
		if($logs[$_[0]]{'logline'} =~ /.*?FAILED LOGIN .*? FROM (.*?) FOR (.*?), /) {
			$evilhost=$1; $targets{$2}=1; $victim=$tmphash{'host'};
			$ep=&to_epoch($logs[$_[0]]{'logline'});
			$newep=$ep+1;
			for($i=$_[0];($ep-$newep)<$time_frame && $i>=0;$i--) {
			    if ($ep >= $newep) {
				%tmphash=&head_parse($logs[$i]{'logline'});
				if($tmphash{'process'} eq 'syslog' && $tmphash{'host'} eq $victim) {
					if($logs[$i]{'logline'}=~/.*?FAILED LOGIN .*? FROM (.*?) FOR (.*?), /) {
						$targets{$2}=1; $numtries ++;
					}
				}
			    }
			if ($logs[$i-1]{'logfile'} eq $logs[$_[0]]{'logfile'})
			    { $newep=&to_epoch($logs[$i-1]{'logline'}); }
			}
			@keys=keys(%targets);
			for($i=0;$keys[$i];$i++) {
				$keys.="$keys[$i],";
			}
			chop($keys);
			if($numtries > $max_failed_attempts) {
				@retval=($evilhost,$keys);
			}
		}
	}
	return @retval;
}

# This function will check for multiple failed incoming ftp attempts.
# Conditions for trigger: same host has > 2 failed ftp attempts 
# within a 20 second time frame.
#
# To modify the trigger conditions, change the $max_failed_attempts and
# $time_frame variables.
sub mod_failed_multiple_ftps {
	(my @retval=());
	(my %tmphash=());
	(my $evilhost="");
	(my $victim="");
	(my %targets=());
	(my $ep="");
	(my $newep="");
	(my $numtries=1);
	(my @keys=());
	(my $keys="");
	(my $i);

	(my $max_failed_attempts = 2);
	(my $time_frame = 20);

	%tmphash=&head_parse($logs[$_[0]]{'logline'});

	if($tmphash{'process'} eq 'ftpd') {
		if($logs[$_[0]]{'logline'}=~/.*?failed login from (.*?), (.*?)$/){ 
			$evilhost=$1; $targets{$2}=1; $victim=$tmphash{'host'};
			$ep=&to_epoch($logs[$_[0]]{'logline'})+1;
			$newep=$ep+1;
			for($i=$_[0];($ep-$newep)<$time_frame && $i>=0;$i--) {
			    if ($ep >= $newep) {
				%tmphash=&head_parse($logs[$i]{'logline'});
				if($tmphash{'process'} eq 'ftpd' && $tmphash{'host'} eq $victim) {
					if($logs[$i]{'logline'}=~/.*?failed login from (.*?), (.*?)$/){
						$targets{$2}=1; $numtries++;
					} 
			        } 
			    }
			if ($logs[$i-1]{'logfile'} eq $logs[$_[0]]{'logfile'})
			    { $newep=&to_epoch($logs[$i-1]{'logline'}); }
			}
			@keys=keys(%targets);
			for($i=0;$keys[$i];$i++) {
				$keys.="$keys[$i],";
			}
			chop($keys);
			if($numtries > $max_failed_attempts) {
				@retval=($evilhost,$keys);
			}
		}
	}
	return @retval;
}



# This function is intended to do some basic checking for a "strobe".
# The way we check for this is that when we get a for a particular
# service, we check the previous 5 seconds worth of logs and 
# see if there have been > 2 different services accessed from the 
# same host
#
# To change the trigger conditions, change the $max_services and 
# $time_frame variables.
sub mod_strobe_check {
	(my %tmphash=());
	(my %services=());
	(my $evilhost="");
	(my $victim="");
	(my @retval=());
	(my $keys="");
	(my @keys=());
	(my $i);

	(my $max_services = 2);
	(my $time_frame = 5);

	$logs[$_[0]]{'logline'}=~/^.*?from (.*?)$/i;
	$evilhost=$1;

	%tmphash=&head_parse($logs[$_[0]]{'logline'});
	$services{$tmphash{'process'}}=1;
	$victim=$tmphash{'host'};

	$ep=&to_epoch($logs[$_[0]]{'logline'});
	$newep=&to_epoch($logs[$_[0]-1]{'logline'});
	for($i=$_[0];($ep-$newep)<$time_frame && $i>=0;$i--) {
		if($logs[$i]{'logfile'} eq "/var/log/secure"){ 
			%tmphash=&head_parse($logs[$i]{'logline'});
			$logs[$_[0]]{'logline'}=~/^.*?from (.*?)$/i;
			if($1 eq $evilhost && $tmphash{'host'} eq $victim) {
				$services{$tmphash{'process'}}=1;
			}
		}
		$newep=&to_epoch($logs[$i-1]{'logline'});
	}

	@keys=keys(%services);
	for($i=0;$keys[$i];$i++) {
		$keys.="$keys[$i] ";
	}
	if(($#keys+1) > $max_services) {
		@retval=($evilhost,$keys);
	}
	return @retval;
}

1;
